
# function to generate one- and multi-trait data ---------------------------------------
generate_Tdata <-function(n_species,n_traits, correlation = 0, b){
  # n_species = number of species; n_traits = number of traits; 
  # correlation = AR(1) autocorrelation between subsequent variables
  # b = coefficient of the linear combination (c in the paper)
  # generate n_traits correlated normally distibuted trait variables in Tdat
  # a linear combination of these  Tdat%*%b  in T
  # rescale b and T so that T ~ N(0,1), 
  # value: list with Tdat, T and coefficients (rescaled b)
  ii <- 1:n_traits
  Sigma <-  correlation^abs(outer(ii,ii,  '-'))
  sdT <-  sqrt(b %*% Sigma %*%b)
  coefs <- b/sdT
  Tdat<-MASS::mvrnorm(n_species,mu=rep(0,n_traits),Sigma)
  T <- Tdat %*%coefs
  rownames(T)<- rownames(Tdat)<- paste("spec", seq_len(nrow(Tdat)), sep="")
  return(list(T = T, Tdat= Tdat, coefficients = coefs))
}


# function to generate (zero inflated ) negative binomial data ------------
ZInbinom <- function(mu,sigma = 1, nu= 0){ 
  L <- rnbinom(length(mu), size = 1/sigma, mu = mu)
  # variance = mu + mu^2/size = mu + sigma * mu^2
  L <- ifelse(runif(length(mu)) < nu,  0,L)
  matrix(L,nrow = nrow(mu),ncol = ncol(mu))   
}

# function to generate (zero inflated ) poisson data ------------
ZIpoisson <- function(mu, nu= 0){ 
  L <- rpois(length(mu), lambda = mu)
  L <- ifelse(runif(length(mu)) < nu,  0,L)
  matrix(L,nrow = nrow(mu),ncol = ncol(mu))   
}



# function to generate data according to the Gaussian response model Eqn A1.1 and y~NegBin-----------------------------------------------
generate_community_Gaussian_response <- function(tolerance=2,E,T,n_species = length(T),n_communities = length(E), mu_max = 10,  distribution = "negbin", sigma = 1, nu= 0, fct = 0.8, max.try = 5){
  # using outer instead of a for loop
  # tolerance sets the maximum width of the random sigma_j in the paper (set to 2 in the paper)
  # with checks on empty rows and columns
  # the repeat loop is to ensure that the final number of  species and sites is within fct (0.8) of the number asked for in  argument
    i.try <- 0
  repeat {
    i.try <- i.try + 1
    # one trait, one environmental variable
    E <- c(E)
    T <- c(T)
    h<-runif(n_species,min=0.3,max=1)
    sigma<-runif(n_species)*tolerance
    mu <- outer(rep(mu_max,n_communities),h) * exp(outer(c(E),c(T),FUN = "-")^2 / outer(rep(1,n_communities),-2*sigma^2))+1.e-10
    L <- switch( distribution,
                 poisson = ZIpoisson(mu,nu=nu),
                 binomial = matrix(rbinom(prod(dim(mu)),1, mu),nrow = nrow(mu),ncol = ncol(mu)),
                 negbin = ZInbinom(mu = mu, sigma= sigma, nu = 0),
                 ZINBI =  ZInbinom(mu = mu, sigma=sigma, nu =nu)
    )    
    n_species_c<-sum(colSums(L)!=0) # _c for check
    n_communities_c<-sum(rowSums(L)!=0)
    if ((n_species_c >= fct * n_species) && (n_communities_c >=fct *n_communities || i.try > max.try)){break}
  }
  rownames(L)<- paste("site", seq_len(nrow(L)), sep="")
  colnames(L)<- paste("spec",seq_len(ncol(L)),sep="")
  return(L)  # L = Y in the paper
}


# function to generate data: RC-model/ loglinear model with y~NegBin eqn A.3--------------------
generate_RC_community <- function(E,T, coefs=c(0,0.2,0,0.2,0), sigma_epsilon_ij = 0.2, mu0 =30,
                                  coefsE=c(0.05, -0.1, 0.1),  coefsx=c(0,0,0),
                                  coefsT=c(0.05, -0.1, 0.1),  coefsz=c(0,0,0),
                                  distribution = "negbin", link.mu = "log",
                                  sigma = 1, nu = 0,
                                  fct = 0.8, max.try = 5){
  #
  # one environmental variable E, one trait T
  # main effects:
  #    row and column effects functions 
  #       f_row and f_col: a polynomial in E, T,x and z respectively
  #           d1*E + d2*E^2 + d3*rnorm(n, 0,1), with coefsE = c(d1,d2,d3)
  #           d1*T + d2*T^2 + d3*rnorm(n, 0,1), with coefsT = c(d1,d2,d3)
  #           d1*x + d2*x^2 + d3*rnorm(n, 0,1), with coefsx = c(d1,d2,d3)
  #           d1*z + d2*z^2 + d3*rnorm(n, 0,1), with coefsz = c(d1,d2,d3)
  #           added noise if d3 is nonzero
  # interaction effects
  #        b1*T*E + b2*z*E + b3*T*x + b4*x*z+ b5*xx*zz eps with coefs = c(b1,b2,b3,b4)
  #        with z ~ N(0,1), x~N(0,1), zz ~ N(0,1), xx~N(0,1)and eps~sigma_epsilon_ij*N(0,1)
  #        Note:  trait-environment association only if b1 !=0
  #        Note:  confounding random effects if coefs[2:3] are non-zero
  # 
  # dist ="negbin", "ZINBI" , "pois", "binomial" 
  # link =  link function "log", "logit" 
  # if center. = TRUE then the interaction is double centered
  # the repeat loop is to ensure that the final number of  species and sites is within fct (0.8) of the number asked for in  argument

  # The equi-tolerance versions of the Gaussian models
  # of Peres-Neto, Dray & ter Braak (2016, Ecography http://dx.doi.org/10.1111/ecog.02302) and
  # ter Braak, Peres-Neres & Dray (2016, PeerJ) can be simulated
  # by setting coefsE-coefsz and coef as follows:
  # The environmentally structured, random trait case corresponds to
  # coefsE=c(0,-0.5/tolerance^2, 0);  coefsx=c(0,0,0)
  # coefsT=c(0,0, 0);  coefsz=c(0,-0.5/tolerance^2,0)
  # The trait structured, random environment case corresponds to
  # coefsE=c(0,0, 0);  coefsx=c(0,-0.5/tolerance^2,0)
  # coefsT=c(0,-0.5/tolerance^2, 0);  coefsz=c(0,0,0)
  # The both random case corresponds to
  # coefsE=c(0,0, 0);  coefsx=c(0,-0.5/tolerance^2,0)
  # coefsT=c(0,0, 0);  coefsz=c(0,-0.5/tolerance^2,0)
  
  #  local functions: marginal effects
  f_row <- function(E, coefsE){ 
    effects <- coefsE[1]*E + coefsE[2]*E*E + coefsE[3]*rnorm(length(E))
    effects
  }
  f_col <- function(T, coefsT){f_row(T,coefsE=coefsT)}
  # end local functions
  E <- c(E);
  T <- c(T)
  n_communities <- length(E)
  n_species <- length(T)
  i.try <- 1
  if (length(coefs)==3) coefs <- c(coefs,0,0)
  if (length(coefs)==4) coefs <- c(coefs,0)
  repeat {
    i.try <- i.try + 1
    x <-  rnorm(length(E)); z <-  rnorm(length(T))
    xx <-  rnorm(length(E)); zz <-  rnorm(length(T))
    mu.interaction <- 
      coefs[1] * outer(E,T,FUN = "*") +
      coefs[2] * outer(E,z,FUN = "*") +
      coefs[3] * outer(x,T,FUN = "*") + 
      coefs[4] * outer(x,z,FUN = "*") + 
      coefs[5] * outer(xx,zz,FUN = "*") + 
      sigma_epsilon_ij * matrix(rnorm(n_communities*n_species), nrow =n_communities )
    mu.marginal <- outer(rep(0,n_communities), f_col(T,coefsT), FUN="+") +
      outer(f_row(E,coefsE), rep(0,n_species), FUN="+") +
      outer(rep(0,n_communities), f_col(z,coefsz), FUN="+") +
      outer(f_row(x,coefsx), rep(0,n_species), FUN="+") 
    mu.lp <- mu.marginal + mu.interaction
    # for numerical stability we needed to limit mu.lp
    mu.lp <- ifelse(mu.lp > 10, matrix(10, nrow=nrow(mu.lp),ncol=ncol(mu.lp)), mu.lp)
    mu.lp <- ifelse(mu.lp < -10,matrix(-10, nrow=nrow(mu.lp),ncol=ncol(mu.lp)), mu.lp)           
    if (mu0 > 0.9 && (distribution =="binomial"|| link.mu =="logit"))  mu0 <- 0.5 # 
    mu <- switch(link.mu,
                 log =  mu0 * exp(mu.lp) ,
                 logit = 1/(1 + exp(-( mu.lp + log(mu0/(1-m0)))) )
    )
    L <- switch( distribution,
                 poisson = ZIpoisson(mu,nu=nu),
                 binomial = matrix(rbinom(prod(dim(mu)),1, mu),nrow = nrow(mu),ncol = ncol(mu)),
                 negbin = ZInbinom(mu = mu, sigma= sigma, nu = 0),
                 ZINBI =  ZInbinom(mu = mu, sigma=sigma, nu =nu)
    )
    nspecies_c <- sum(colSums(L)!=0) # _c for check
    ncommunities_c <- sum(rowSums(L)!=0)
    if ((nspecies_c >= fct * n_species) && (ncommunities_c >=fct *n_communities || i.try> max.try)){break}
  }
  rownames(L)<- paste("site", seq_len(nrow(L)), sep="")
  colnames(L)<- paste("spec",seq_len(ncol(L)),sep="")
  return(L) # L = Y in the paper
}

